// RegLib.mqh -     .
// Copyright  2009,  .

#import "advapi32.dll"

    int RegCreateKeyExA( int hKey, string lpSubKey, int Reserved, int lpClass, int dwOptions, int samDesired, int lpSecurityAttributes, int phkResult[], int lpdwDisposition[] );
    int RegOpenKeyExA( int hKey, string lpSubKey, int ulOptions, int samDesired, int phkResult[] );    
    int RegCloseKey( int hKey );
    int RegQueryValueExA( int hKey, string lpValueName, int lpReserved, int lpType[], string lpData, int lpcbData[] );
    int RegSetValueExA( int hKey, string lpValueName, int Reserved, int dwType, string lpData, int cbData );


#define HKEY_CLASSES_ROOT           0x80000000
#define HKEY_CURRENT_USER           0x80000001
#define HKEY_LOCAL_MACHINE          0x80000002
#define HKEY_USERS                  0x80000003
#define HKEY_PERFORMANCE_DATA       0x80000004
#define HKEY_PERFORMANCE_TEXT       0x80000050
#define HKEY_PERFORMANCE_NLSTEXT    0x80000060
#define HKEY_CURRENT_CONFIG         0x80000005
#define HKEY_DYN_DATA               0x80000006

// dwOptions
#define REG_OPTION_BACKUP_RESTORE   0x00000004
#define REG_OPTION_VOLATILE         0x00000001
#define REG_OPTION_NON_VOLATILE     0x00000000

#define ERROR_SUCCESS                  0
#define ERROR_FILE_NOT_FOUND           2
#define ERROR_ACCESS_DENIED            5
#define ERROR_INVALID_HANDLE           6
#define ERROR_INVALID_PARAMETER       87
#define ERROR_CALL_NOT_IMPLEMENTED   120
#define ERROR_INSUFFICIENT_BUFFER    122
#define ERROR_MORE_DATA              234
#define ERROR_NO_MORE_ITEMS          259
#define ERROR_CANTOPEN              1011
#define ERROR_CANTREAD              1012
#define ERROR_CANTWRITE             1013
#define ERROR_REGISTRY_RECOVERED    1014
#define ERROR_REGISTRY_CORRUPT      1015
#define ERROR_REGISTRY_IO_FAILED    1016
#define ERROR_NOT_REGISTRY_FILE     1017
#define ERROR_KEY_DELETED           1018

int KEY_QUERY_VALUE; //    
int KEY_SET_VALUE; //  ,      
int KEY_CREATE_SUB_KEY;
int KEY_ENUMERATE_SUB_KEYS; //     
int KEY_NOTIFY;
int KEY_CREATE_LINK;
int KEY_WOW64_32KEY;
int KEY_WOW64_64KEY;
int KEY_WOW64_RES;

int KEY_READ; //   
int KEY_WRITE;
int KEY_EXECUTE;
int KEY_ALL_ACCESS; //     

#define REG_CREATED_NEW_KEY         0x00000001   //      
#define REG_OPENED_EXISTING_KEY     0x00000002   //   

#define REG_NONE                       0   // No value type
#define REG_SZ                         1   // Unicode nul terminated string
#define REG_EXPAND_SZ                  2   // Unicode nul terminated string
                                           // (with environment variable references)
#define REG_BINARY                     3   // Free form binary
#define REG_DWORD                      4   // 32-bit number
#define REG_DWORD_LITTLE_ENDIAN        4   // 32-bit number (same as REG_DWORD)
#define REG_DWORD_BIG_ENDIAN           5   // 32-bit number
#define REG_LINK                       6   // Symbolic Link (unicode)
#define REG_MULTI_SZ                   7   // Multiple Unicode strings
#define REG_RESOURCE_LIST              8   // Resource list in the resource map
#define REG_FULL_RESOURCE_DESCRIPTOR   9   // Resource list in the hardware description
#define REG_RESOURCE_REQUIREMENTS_LIST 10
#define REG_QWORD                      11  // 64-bit number
#define REG_QWORD_LITTLE_ENDIAN        11  // 64-bit number (same as REG_QWORD)


void InitRegDefines()
{
   int STANDARD_RIGHTS_READ = 0x00020000;
   int STANDARD_RIGHTS_WRITE = 0x00020000;
   int SYNCHRONIZE = 0x00100000;
   int STANDARD_RIGHTS_ALL = 0x001F0000;

   KEY_QUERY_VALUE = 0x0001;
   KEY_SET_VALUE = 0x0002;
   KEY_CREATE_SUB_KEY = 0x0004;
   KEY_ENUMERATE_SUB_KEYS = 0x0008;
   KEY_NOTIFY = 0x0010;
   KEY_CREATE_LINK = 0x0020;
   KEY_WOW64_32KEY = 0x0200;
   KEY_WOW64_64KEY = 0x0100;
   KEY_WOW64_RES = 0x0300;

   KEY_READ = ( ( STANDARD_RIGHTS_READ | KEY_QUERY_VALUE | KEY_ENUMERATE_SUB_KEYS | KEY_NOTIFY ) & ( ~SYNCHRONIZE ) );
   KEY_WRITE = ( ( STANDARD_RIGHTS_WRITE | KEY_SET_VALUE | KEY_CREATE_SUB_KEY ) & ( ~SYNCHRONIZE ) );
   KEY_EXECUTE = ( ( KEY_READ ) & ( ~SYNCHRONIZE ) );
   KEY_ALL_ACCESS = ( ( STANDARD_RIGHTS_ALL | KEY_QUERY_VALUE | KEY_SET_VALUE | KEY_CREATE_SUB_KEY | KEY_ENUMERATE_SUB_KEYS | KEY_NOTIFY | KEY_CREATE_LINK ) & ( ~SYNCHRONIZE ) );
}

string GetErrorString( int ErrorCode )
{
   switch( ErrorCode )
   {
      case ERROR_SUCCESS:
         return ( "ERROR_SUCCESS" );

      case ERROR_FILE_NOT_FOUND:
         return ( "ERROR_FILE_NOT_FOUND" );

      case ERROR_ACCESS_DENIED:
         return ( "ERROR_ACCESS_DENIED" );

      case ERROR_INVALID_HANDLE:
         return ( "ERROR_INVALID_HANDLE" );

      case ERROR_INVALID_PARAMETER:
         return ( "ERROR_INVALID_PARAMETER" );

      case ERROR_CALL_NOT_IMPLEMENTED:
         return ( "ERROR_CALL_NOT_IMPLEMENTED" );

      case ERROR_INSUFFICIENT_BUFFER:
         return ( "ERROR_INSUFFICIENT_BUFFER" );

      case ERROR_MORE_DATA:
         return ( "ERROR_MORE_DATA" );

      case ERROR_NO_MORE_ITEMS:
         return ( "ERROR_NO_MORE_ITEMS" );

      case ERROR_CANTOPEN:
         return ( "ERROR_CANTOPEN" );

      case ERROR_CANTREAD:
         return ( "ERROR_CANTREAD" );

      case ERROR_CANTWRITE:
         return ( "ERROR_CANTWRITE" );

      case ERROR_REGISTRY_RECOVERED:
         return ( "ERROR_REGISTRY_RECOVERED" );

      case ERROR_REGISTRY_CORRUPT:
         return ( "ERROR_REGISTRY_CORRUPT" );

      case ERROR_REGISTRY_IO_FAILED:
         return ( "ERROR_REGISTRY_IO_FAILED" );

      case ERROR_NOT_REGISTRY_FILE:
         return ( "ERROR_NOT_REGISTRY_FILE" );

      case ERROR_KEY_DELETED:
         return ( "ERROR_KEY_DELETED" );
   }
}

int CreateKey( int hHomeKey, string KeyName )
{
   // Handle  
   int Key[ 1 ];

   //  
   Key[ 0 ] = 0;

   //  
   int lpdwDisposition[ 1 ];

   //  API
   int lres = RegCreateKeyExA( hHomeKey, KeyName, 0, 0, REG_OPTION_VOLATILE, KEY_ALL_ACCESS, 0, Key, lpdwDisposition );

   //  
   if ( lres != ERROR_SUCCESS )
   {
      Print( "Error in RegCreateKeyExA(): ", GetErrorString( lres ) );
      return( Key[ 0 ] );
   }

   //    
   switch ( lpdwDisposition[ 0 ] )
   {
      case REG_CREATED_NEW_KEY:
         Print( "RegCreateKeyExA(): ", "   ." );
         break;

      case REG_OPENED_EXISTING_KEY:
         Print( "RegCreateKeyExA(): ", "  ." );
         break;
   }

   return ( Key[ 0 ] );
}

int OpenKey( int hHomeKey, string KeyName )
{
   // Handle  
   int Key[ 1 ];

   //  
   Key[ 0 ] = 0;

   //  API
   int lres = RegOpenKeyExA( int hHomeKey, KeyName, 0, KEY_ALL_ACCESS, Key );

   //  
   if ( lres != ERROR_SUCCESS )
   {
      Print( "Error in RegOpenKeyExA(): ", GetErrorString( lres ) );
   }

   return( Key[ 0 ] );
}

void CloseKey( int hKey )
{
   //  API
   int lres = RegCloseKey( hKey );

   //  
   if ( lres != ERROR_SUCCESS )
   {
      Print( "Error in RegCloseKey(): ", GetErrorString( lres ) );
   }
}

void SetStringValue( int hKey, string ValueName, string ValueData )
{
   //   - 
   int dwType = REG_SZ;

   //   
   int Len = StringLen( ValueData );

   //   0
   ValueData = StringSetChar( ValueData, Len, '\x00' );

   //  API
   int lres = RegSetValueExA( hKey, ValueName, 0, dwType, ValueData, Len + 1 );

   //  
   if ( lres != ERROR_SUCCESS )
   {
      Print( "Error in RegSetValueExA(): ", GetErrorString( lres ) );
   }
}

string GetStringValue( int hKey, string ValueName )
{
   //   
   int lpType[ 1 ];

   //    
   string lpData = "###############################################################################################################################################################################################################################################################";

   //  
   int lpcbData[ 1 ];
   lpcbData[ 0 ] = 255;

   int p; //      

   //  API
   int lres = RegQueryValueExA( hKey, ValueName, 0, lpType, lpData, lpcbData );
   //   lpcbData[ 0 ]   

   //  
   if ( lres != ERROR_SUCCESS )
   {
      Print( "Error in RegQueryValueExA(): ", GetErrorString( lres ) );
      return( "" );
   }

   if ( lpType[ 0 ] == REG_SZ || lpType[ 0 ] == REG_EXPAND_SZ )
   {   
       return( StringSubstr( lpData, 0, lpcbData[ 0 ] - 1 ) );
   }
/*   else if ( lpType[ 0 ] == REG_MULTI_SZ )
   {    
      for( p = lpcbData[ 0 ]; 0 < p; p-- ) if( StringGetChar( lpData, p ) != 0 ) break; 
      lpData = StringSubstr( lpData, 0, p );
      for( p = StringLen( lpData ); 0 < p; p-- ) if( StringGetChar( lpData, p ) == 0 ) StringSetChar( lpData, p, 10 ); //     

      return( lpData );
  }*/

   return( "" );
}